import json
import unittest
import pyyoutube.models as models


class SubscriptionModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/subscriptions/"

    with open(BASE_PATH + "snippet.json", "rb") as f:
        SNIPPETS = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "contentDetails.json", "rb") as f:
        CONTENT_DETAILS = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "subscriberSnippet.json", "rb") as f:
        SUBSCRIBER_SNIPPET = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "subscription.json", "rb") as f:
        SUBSCRIPTION_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "resp.json", "rb") as f:
        SUBSCRIPTION_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testSubscriptionSnippet(self) -> None:
        m = models.SubscriptionSnippet.from_dict(self.SNIPPETS)

        self.assertEqual(m.channelId, "UCNvMBmCASzTNNX8lW3JRMbw")
        self.assertEqual(m.resourceId.channelId, "UCQ7dFBzZGlBvtU2hCecsBBg")
        self.assertEqual(
            m.thumbnails.default.url,
            "https://yt3.ggpht.com/s88-c-k-no-mo-rj-c0xffffff/photo.jpg",
        )

    def testSubscriptionContentDetails(self) -> None:
        m = models.SubscriptionContentDetails.from_dict(self.CONTENT_DETAILS)

        self.assertEqual(m.totalItemCount, 2)
        self.assertEqual(m.activityType, "all")

    def testSubscriptionSubscriberSnippet(self) -> None:
        m = models.SubscriptionSubscriberSnippet.from_dict(self.SUBSCRIBER_SNIPPET)

        self.assertEqual(m.title, "kun liu")
        self.assertEqual(
            m.thumbnails.default.url,
            "https://yt3.ggpht.com/s88-c-k-no-mo-rj-c0xffffff/photo.jpg",
        )

    def testSubscription(self) -> None:
        m = models.Subscription.from_dict(self.SUBSCRIPTION_INFO)

        self.assertEqual(m.id, "zqShTXi-2-Rya5uUxEp3ZsPI3fZrFQnSXNQCwvHBGGo")
        self.assertEqual(m.snippet.title, "ikaros-life")
        self.assertEqual(m.contentDetails.totalItemCount, 2)
        self.assertEqual(m.subscriberSnippet.title, "kun liu")

    def testSubscriptionResponse(self) -> None:
        m = models.SubscriptionListResponse.from_dict(self.SUBSCRIPTION_RESPONSE)

        self.assertEqual(m.nextPageToken, "CAUQAA")
        self.assertEqual(m.pageInfo.totalResults, 16)
        self.assertEqual(len(m.items), 5)



import json
import unittest

import pyyoutube.models as models


class ChannelSectionModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/channel_sections/"

    with open(BASE_PATH + "channel_section_info.json", "rb") as f:
        CHANNEL_SECTION_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_section_response.json", "rb") as f:
        CHANNEL_SECTION_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testChannelSection(self) -> None:
        m = models.ChannelSection.from_dict(self.CHANNEL_SECTION_INFO)

        self.assertEqual(m.id, "UC_x5XG1OV2P6uZZ5FSM9Ttw.e-Fk7vMPqLE")
        self.assertEqual(m.snippet.type, "multipleChannels")
        self.assertEqual(len(m.contentDetails.channels), 16)

    def testChannelSectionResponse(self) -> None:
        m = models.ChannelSectionResponse.from_dict(self.CHANNEL_SECTION_RESPONSE)

        self.assertEqual(m.kind, "youtube#channelSectionListResponse")
        self.assertEqual(len(m.items), 10)



import json
import unittest

import pyyoutube.models as models


class AbuseReasonModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/abuse_report_reason/"

    with open(BASE_PATH + "abuse_reason.json", "rb") as f:
        ABUSE_REASON = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "abuse_reason_res.json", "rb") as f:
        ABUSE_REASON_RES = json.loads(f.read().decode("utf-8"))

    def testAbuseReason(self) -> None:
        m = models.VideoAbuseReportReason.from_dict(self.ABUSE_REASON)

        self.assertEqual(m.id, "N")
        self.assertEqual(m.snippet.label, "Sex or nudity")
        self.assertEqual(len(m.snippet.secondaryReasons), 3)

    def testAbuseReasonResponse(self) -> None:
        m = models.VideoAbuseReportReasonListResponse.from_dict(self.ABUSE_REASON_RES)

        self.assertEqual(m.kind, "youtube#videoAbuseReportReasonListResponse")
        self.assertEqual(len(m.items), 3)



import json
import unittest

import responses
from requests import HTTPError

import pyyoutube


class TestOAuthApi(unittest.TestCase):
    BASE_PATH = "testdata/apidata/"

    with open(BASE_PATH + "access_token.json", "rb") as f:
        ACCESS_TOKEN_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "user_profile.json", "rb") as f:
        USER_PROFILE_INFO = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(client_id="xx", client_secret="xx")

    def testInitApi(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            pyyoutube.Api()

    def testOAuth(self) -> None:
        url, statue = self.api.get_authorization_url()
        self.assertEqual(statue, "PyYouTube")

        redirect_response = (
            "https://localhost/?state=PyYouTube&code=code"
            "&scope=profile+https%3A%2F%2Fwww.googleapis.com%2Fauth%2Fuserinfo.profile#"
        )

        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.refresh_token()

        with responses.RequestsMock() as m:
            m.add(
                "POST", self.api.EXCHANGE_ACCESS_TOKEN_URL, json=self.ACCESS_TOKEN_INFO
            )
            token = self.api.generate_access_token(
                authorization_response=redirect_response,
            )
            self.assertEqual(token.access_token, "access_token")
            token_origin = self.api.generate_access_token(
                authorization_response=redirect_response, return_json=True
            )
            self.assertEqual(token_origin["access_token"], "access_token")

            refresh_token = self.api.refresh_token()
            self.assertEqual(refresh_token.access_token, "access_token")
            refresh_token_origin = self.api.refresh_token(return_json=True)
            self.assertEqual(refresh_token_origin["refresh_token"], "refresh_token")

            api = pyyoutube.Api(client_id="xx", client_secret="xx")
            refresh_token = api.refresh_token(refresh_token="refresh_token")
            self.assertEqual(refresh_token.refresh_token, "refresh_token")

    def testGetProfile(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_profile()

        self.api._access_token = "access_token"
        with responses.RequestsMock() as m:
            m.add("GET", self.api.USER_INFO_URL, json=self.USER_PROFILE_INFO)
            profile = self.api.get_profile()
            self.assertEqual(profile.given_name, "kun")

            profile_origin = self.api.get_profile(return_json=True)
            self.assertEqual(profile_origin["given_name"], "kun")

        with responses.RequestsMock() as m:
            m.add("GET", self.api.USER_INFO_URL, body=HTTPError("Exception"))
            with self.assertRaises(pyyoutube.PyYouTubeException):
                self.api.get_profile()



import json
import unittest

import pyyoutube.models as models


class CategoryModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/categories/"

    with open(BASE_PATH + "video_category_info.json", "rb") as f:
        VIDEO_CATEGORY_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_category_response.json", "rb") as f:
        VIDEO_CATEGORY_RESPONSE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "guide_category_info.json", "rb") as f:
        GUIDE_CATEGORY_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "guide_category_response.json", "rb") as f:
        GUIDE_CATEGORY_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testVideoCategory(self) -> None:
        m = models.VideoCategory.from_dict(self.VIDEO_CATEGORY_INFO)
        self.assertEqual(m.id, "17")
        self.assertEqual(m.snippet.title, "Sports")

    def testVideoCategoryListResponse(self) -> None:
        m = models.VideoCategoryListResponse.from_dict(self.VIDEO_CATEGORY_RESPONSE)
        self.assertEqual(m.kind, "youtube#videoCategoryListResponse")
        self.assertEqual(len(m.items), 1)
        self.assertEqual(m.items[0].id, "17")



import unittest

from pyyoutube import youtube_utils as utils
from pyyoutube.error import PyYouTubeException


class UtilsTest(unittest.TestCase):
    def testDurationConvert(self):
        duration = "PT14H23M42S"
        self.assertEqual(utils.get_video_duration(duration), 51822)

        duration = "PT14H23M42"
        with self.assertRaises(PyYouTubeException):
            utils.get_video_duration(duration)



from .activity import (
    Activity,
    ActivityContentDetails,
    ActivityListResponse,
    ActivitySnippet,
)
from .auth import AccessToken, UserProfile
from .caption import Caption, CaptionListResponse, CaptionSnippet
from .category import (
    VideoCategory,
    VideoCategoryListResponse,
)
from .channel import (
    Channel,
    ChannelBrandingSetting,
    ChannelContentDetails,
    ChannelListResponse,
    ChannelSnippet,
    ChannelStatistics,
    ChannelStatus,
    ChannelTopicDetails,
)
from .channel_section import (
    ChannelSection,
    ChannelSectionContentDetails,
    ChannelSectionSnippet,
    ChannelSectionResponse,
)
from .comment import (
    Comment,
    CommentListResponse,
    CommentSnippet,
    CommentThread,
    CommentThreadListResponse,
    CommentThreadReplies,
    CommentThreadSnippet,
)
from .playlist import (
    Playlist,
    PlaylistContentDetails,
    PlaylistListResponse,
    PlaylistSnippet,
    PlaylistStatus,
)
from .playlist_item import (
    PlaylistItem,
    PlaylistItemContentDetails,
    PlaylistItemListResponse,
    PlaylistItemSnippet,
    PlaylistItemStatus,
)
from .subscription import (
    Subscription,
    SubscriptionContentDetails,
    SubscriptionListResponse,
    SubscriptionSnippet,
    SubscriptionSubscriberSnippet,
)
from .video import (
    Video,
    VideoContentDetails,
    VideoListResponse,
    VideoSnippet,
    VideoStatistics,
    VideoStatus,
    VideoTopicDetails,
)
from .i18n import (
    I18nRegion,
    I18nRegionListResponse,
    I18nLanguage,
    I18nLanguageListResponse,
)

from .video_abuse_report_reason import (
    VideoAbuseReportReason,
    VideoAbuseReportReasonListResponse,
)
from .search_result import (
    SearchResultId,
    SearchResultSnippet,
    SearchResult,
    SearchListResponse,
)
from .member import Member, MemberListResponse
from .memberships_level import MembershipsLevel, MembershipsLevelListResponse

__all__ = [
    "AccessToken",
    "Activity",
    "ActivityContentDetails",
    "ActivityListResponse",
    "ActivitySnippet",
    "Caption",
    "CaptionListResponse",
    "CaptionSnippet",
    "Channel",
    "ChannelBrandingSetting",
    "ChannelContentDetails",
    "ChannelListResponse",
    "ChannelSnippet",
    "ChannelStatistics",
    "ChannelStatus",
    "ChannelTopicDetails",
    "ChannelSection",
    "ChannelSectionContentDetails",
    "ChannelSectionSnippet",
    "ChannelSectionResponse",
    "Comment",
    "CommentListResponse",
    "CommentSnippet",
    "CommentThread",
    "CommentThreadListResponse",
    "CommentThreadReplies",
    "CommentThreadSnippet",
    "Playlist",
    "PlaylistContentDetails",
    "PlaylistItem",
    "PlaylistItemContentDetails",
    "PlaylistItemListResponse",
    "PlaylistItemSnippet",
    "PlaylistItemStatus",
    "PlaylistItemListResponse",
    "Comment",
    "CommentSnippet",
    "CommentListResponse",
    "CommentThread",
    "CommentThreadSnippet",
    "CommentThreadReplies",
    "CommentThreadListResponse",
    "SearchResult",
    "SearchListResponse",
    "SearchResultSnippet",
    "SearchResultId",
    "PlaylistListResponse",
    "PlaylistSnippet",
    "PlaylistStatus",
    "Subscription",
    "SubscriptionContentDetails",
    "SubscriptionListResponse",
    "SubscriptionSnippet",
    "SubscriptionSubscriberSnippet",
    "UserProfile",
    "Video",
    "VideoCategory",
    "VideoCategoryListResponse",
    "VideoContentDetails",
    "VideoListResponse",
    "VideoSnippet",
    "VideoStatistics",
    "VideoStatus",
    "VideoTopicDetails",
    "I18nLanguage",
    "I18nLanguageListResponse",
    "I18nRegion",
    "I18nRegionListResponse",
    "VideoAbuseReportReason",
    "VideoAbuseReportReasonListResponse",
    "Member",
    "MemberListResponse",
    "MembershipsLevel",
    "MembershipsLevelListResponse",
]



import unittest

import pyyoutube
from pyyoutube.utils.params_checker import enf_comma_separated, enf_parts


class ParamCheckerTest(unittest.TestCase):
    def testEnfCommaSeparated(self) -> None:
        self.assertIsNone(enf_comma_separated("id", None))
        self.assertEqual(enf_comma_separated("id", "my_id"), "my_id")
        self.assertEqual(enf_comma_separated("id", "id1,id2"), "id1,id2")
        self.assertEqual(enf_comma_separated("id", ["id1", "id2"]), "id1,id2")
        self.assertEqual(enf_comma_separated("id", ("id1", "id2")), "id1,id2")
        self.assertTrue(enf_comma_separated("id", {"id1", "id2"}))

        with self.assertRaises(pyyoutube.PyYouTubeException):
            enf_comma_separated("id", 1)
        with self.assertRaises(pyyoutube.PyYouTubeException):
            enf_comma_separated("id", [None, None])

    def testEnfParts(self) -> None:
        self.assertTrue(enf_parts(resource="channels", value=None))
        self.assertTrue(enf_parts(resource="channels", value="id"), "id")
        self.assertTrue(enf_parts(resource="channels", value="id,snippet"))
        self.assertTrue(enf_parts(resource="channels", value=["id", "snippet"]))
        self.assertTrue(enf_parts(resource="channels", value=("id", "snippet")))
        self.assertTrue(enf_parts(resource="channels", value={"id", "snippet"}))

        with self.assertRaises(pyyoutube.PyYouTubeException):
            enf_parts(resource="channels", value=1)

        with self.assertRaises(pyyoutube.PyYouTubeException):
            enf_parts(resource="channels", value="not_part")



import json
import unittest

import responses

import pyyoutube


class ApiActivitiesTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/activities/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/activities"

    with open(BASE_PATH + "activities_by_channel_p1.json", "rb") as f:
        ACTIVITIES_CHANNEL_P1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "activities_by_channel_p2.json", "rb") as f:
        ACTIVITIES_CHANNEL_P2 = json.loads(f.read().decode("utf-8"))

    with open(BASE_PATH + "activities_by_mine_p1.json", "rb") as f:
        ACTIVITIES_MINE_P1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "activities_by_mine_p2.json", "rb") as f:
        ACTIVITIES_MINE_P2 = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")
        self.api_with_access_token = pyyoutube.Api(access_token="token")

    def testGetChannelActivities(self) -> None:
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_activities_by_channel(channel_id="id", parts="id,not_part")

        # test get all activities
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_CHANNEL_P1)
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_CHANNEL_P2)

            res = self.api.get_activities_by_channel(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                parts="id,snippet",
                before="2019-11-1T00:00:00.000Z",
                after="2019-10-1T00:00:00.000Z",
                region_code="US",
                count=None,
            )
            self.assertEqual(len(res.items), 13)

        # test get by page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_CHANNEL_P2)

            res = self.api.get_activities_by_channel(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                parts="id,snippet",
                count=None,
                page_token="CAoQAA",
                return_json=True,
            )
            self.assertEqual(len(res["items"]), 3)

    def testGetMineActivities(self) -> None:
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api_with_access_token.get_activities_by_me(parts="id,not_part")

        # test get all activities
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_MINE_P1)
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_MINE_P2)

            res = self.api_with_access_token.get_activities_by_me(
                parts="id,snippet",
                before="2019-11-1T00:00:00.000Z",
                after="2019-12-1T00:00:00.000Z",
                region_code="US",
                count=None,
            )
            self.assertEqual(len(res.items), 2)

        # test page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ACTIVITIES_MINE_P2)

            res = self.api_with_access_token.get_activities_by_me(
                parts="id,snippet",
                before="2019-11-1T00:00:00.000Z",
                after="2019-12-1T00:00:00.000Z",
                region_code="US",
                page_token="CAEQAA",
                count=None,
                return_json=True,
            )
            self.assertEqual(len(res["items"]), 1)



import json
import unittest

import pyyoutube.models as models


class PlaylistModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/playlists/"

    with open(BASE_PATH + "playlist_content_details.json", "rb") as f:
        CONTENT_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_snippet.json", "rb") as f:
        SNIPPET_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_status.json", "rb") as f:
        STATUS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_info.json", "rb") as f:
        PLAYLIST_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_api_response.json", "rb") as f:
        PLAYLIST_RESPONSE_INFO = json.loads(f.read().decode("utf-8"))

    def testPlayListContentDetails(self) -> None:
        m = models.PlaylistContentDetails.from_dict(self.CONTENT_DETAILS_INFO)

        self.assertEqual(m.itemCount, 4)

    def testPlayListSnippet(self) -> None:
        m = models.PlaylistSnippet.from_dict(self.SNIPPET_INFO)

        self.assertEqual(
            m.string_to_datetime(m.publishedAt).isoformat(), "2019-05-16T18:46:20+00:00"
        )
        self.assertEqual(m.title, "Assistant on Air")
        self.assertEqual(
            m.thumbnails.default.url, "https://i.ytimg.com/vi/D-lhorsDlUQ/default.jpg"
        )
        self.assertEqual(m.localized.title, "Assistant on Air")

    def testPlayListStatus(self) -> None:
        m = models.PlaylistStatus.from_dict(self.STATUS_INFO)

        self.assertEqual(m.privacyStatus, "public")

    def testPlayList(self) -> None:
        m = models.Playlist.from_dict(self.PLAYLIST_INFO)

        self.assertEqual(m.id, "PLOU2XLYxmsIJpufeMHncnQvFOe0K3MhVp")
        self.assertEqual(m.player, None)
        self.assertEqual(m.snippet.title, "Assistant on Air")

    def testPlaylistListResponse(self) -> None:
        m = models.PlaylistListResponse.from_dict(self.PLAYLIST_RESPONSE_INFO)

        self.assertEqual(m.kind, "youtube#playlistListResponse")
        self.assertEqual(m.pageInfo.totalResults, 416)
        self.assertEqual(m.items[0].id, "PLOU2XLYxmsIJpufeMHncnQvFOe0K3MhVp")



import json
import unittest

import pyyoutube.models as models


class MemberModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/members/"

    with open(BASE_PATH + "member_info.json", "rb") as f:
        MEMBER_INFO = json.loads(f.read().decode("utf-8"))

    def testMember(self) -> None:
        m = models.Member.from_dict(self.MEMBER_INFO)

        self.assertEqual(m.kind, "youtube#member")
        self.assertEqual(m.snippet.memberDetails.channelId, "UCa-vrCLQHviTOVnEKDOdetQ")
        self.assertEqual(m.snippet.membershipsDetails.highestAccessibleLevel, "string")


class MembershipLevelModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/members/"

    with open(BASE_PATH + "membership_level.json", "rb") as f:
        MEMBERSHIP_LEVEL_INFO = json.loads(f.read().decode("utf-8"))

    def testMembershipLevel(self) -> None:
        m = models.MembershipsLevel.from_dict(self.MEMBERSHIP_LEVEL_INFO)

        self.assertEqual(m.kind, "youtube#membershipsLevel")
        self.assertEqual(m.snippet.levelDetails.displayName, "high")



import json
import unittest

import pyyoutube.models as models


class SearchResultModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/search_result/"

    with open(BASE_PATH + "search_result_id.json", "rb") as f:
        SEARCH_RES_ID_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "search_result_snippet.json", "rb") as f:
        SEARCH_RES_SNIPPET_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "search_result.json", "rb") as f:
        SEARCH_RES_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "search_result_api_response.json", "rb") as f:
        SEARCH_RES_API_INFO = json.loads(f.read().decode("utf-8"))

    def testSearchResultId(self):
        m = models.SearchResultId.from_dict(self.SEARCH_RES_ID_INFO)
        self.assertEqual(m.kind, "youtube#playlist")

    def testSearchResultSnippet(self):
        m = models.SearchResultSnippet.from_dict(self.SEARCH_RES_SNIPPET_INFO)

        self.assertEqual(m.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw")
        self.assertEqual(
            m.string_to_datetime(m.publishedAt).isoformat(),
            "2016-03-30T16:59:12+00:00",
        )
        self.assertEqual(
            m.thumbnails.default.url, "https://i.ytimg.com/vi/cKxRvEZd3Mw/default.jpg"
        )

    def testSearchResult(self):
        m = models.SearchResult.from_dict(self.SEARCH_RES_INFO)
        self.assertEqual(m.kind, "youtube#searchResult")
        self.assertEqual(m.id.videoId, "fq4N0hgOWzU")

    def testSearchListResponse(self):
        m = models.SearchListResponse.from_dict(self.SEARCH_RES_API_INFO)

        self.assertEqual(m.kind, "youtube#searchListResponse")
        self.assertEqual(m.regionCode, "US")
        self.assertEqual(m.pageInfo.totalResults, 489126)
        self.assertEqual(len(m.items), 5)



"""
    function's params checker.
"""
import logging

from typing import Optional, Union

from pyyoutube.error import ErrorCode, ErrorMessage, PyYouTubeException
from pyyoutube.utils.constants import RESOURCE_PARTS_MAPPING

logger = logging.getLogger(__name__)


def enf_comma_separated(
    field: str,
    value: Optional[Union[str, list, tuple, set]],
):
    """
    Check to see if field's value type belong to correct type.
    If it is, return api need value, otherwise, raise a PyYouTubeException.

    Args:
        field (str):
            Name of the field you want to do check.
        value (str, list, tuple, set, Optional)
            Value for the field.

    Returns:
        Api needed string
    """
    if value is None:
        return None
    try:
        if isinstance(value, str):
            return value
        elif isinstance(value, (list, tuple, set)):
            if isinstance(value, set):
                logging.warning(f"Note: The order of the set is unreliable.")
            return ",".join(value)
        else:
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.INVALID_PARAMS,
                    message=f"Parameter ({field}) must be single str,comma-separated str,list,tuple or set",
                )
            )
    except (TypeError, ValueError):
        raise PyYouTubeException(
            ErrorMessage(
                status_code=ErrorCode.INVALID_PARAMS,
                message=f"Parameter ({field}) must be single str,comma-separated str,list,tuple or set",
            )
        )


def enf_parts(resource: str, value: Optional[Union[str, list, tuple, set]], check=True):
    """
    Check to see if value type belong to correct type, and if resource support the given part.
    If it is, return api need value, otherwise, raise a PyYouTubeException.

    Args:
        resource (str):
            Name of the resource you want to retrieve.
        value (str, list, tuple, set, Optional):
            Value for the part.
        check (bool, optional):
            Whether check the resource properties.

    Returns:
        Api needed part string
    """
    if value is None:
        parts = RESOURCE_PARTS_MAPPING[resource]
    elif isinstance(value, str):
        parts = set(value.split(","))
    elif isinstance(value, (list, tuple, set)):
        parts = set(value)
    else:
        raise PyYouTubeException(
            ErrorMessage(
                status_code=ErrorCode.INVALID_PARAMS,
                message=f"Parameter (parts) must be single str,comma-separated str,list,tuple or set",
            )
        )
    # check parts whether support.
    if check:
        support_parts = RESOURCE_PARTS_MAPPING[resource]
        if not support_parts.issuperset(parts):
            not_support_parts = ",".join(parts.difference(support_parts))
            raise PyYouTubeException(
                ErrorMessage(
                    status_code=ErrorCode.INVALID_PARAMS,
                    message=f"Parts {not_support_parts} for resource {resource} not support",
                )
            )
    return ",".join(parts)



import json
import unittest

import responses

import pyyoutube


class ApiPlaylistItemTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/playlist_items/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/playlistItems"

    with open(BASE_PATH + "playlist_items_single.json", "rb") as f:
        PLAYLIST_ITEM_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_items_multi.json", "rb") as f:
        PLAYLIST_ITEM_INFO_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_items_paged_1.json", "rb") as f:
        PLAYLIST_ITEM_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_items_paged_2.json", "rb") as f:
        PLAYLIST_ITEM_PAGED_2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_items_filter_video.json", "rb") as f:
        PLAYLIST_ITEM_FILTER = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testGetPlaylistItemById(self) -> None:
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_INFO_SINGLE)
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_INFO_MULTI)

            res_by_single_id = self.api.get_playlist_item_by_id(
                playlist_item_id="UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS41NkI0NEY2RDEwNTU3Q0M2",
                parts="id,snippet",
                return_json=True,
            )
            self.assertEqual(
                res_by_single_id["kind"], "youtube#playlistItemListResponse"
            )
            self.assertEqual(
                res_by_single_id["items"][0]["id"],
                "UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS41NkI0NEY2RDEwNTU3Q0M2",
            )

            res_by_multi_id = self.api.get_playlist_item_by_id(
                playlist_item_id=[
                    "UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS41NkI0NEY2RDEwNTU3Q0M2",
                    "UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS4yODlGNEE0NkRGMEEzMEQy",
                ],
                parts=["id", "snippet"],
            )
            self.assertEqual(res_by_multi_id.pageInfo.totalResults, 2)
            self.assertEqual(len(res_by_multi_id.items), 2)
            self.assertEqual(
                res_by_multi_id.items[1].id,
                "UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS4yODlGNEE0NkRGMEEzMEQy",
            )

    def testGetPlaylistItems(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_playlist_items(playlist_id="id", parts="id,not_part")

        # test paged
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_2)

            res_by_playlist = self.api.get_playlist_items(
                playlist_id="PLOU2XLYxmsIKpaV8h0AGE05so0fAwwfTw",
                parts="id,snippet",
                limit=10,
                count=20,
            )
            self.assertEqual(res_by_playlist.kind, "youtube#playlistItemListResponse")
            self.assertEqual(res_by_playlist.pageInfo.totalResults, 13)
            self.assertEqual(len(res_by_playlist.items), 13)
            self.assertEqual(
                res_by_playlist.items[0].id,
                "UExPVTJYTFl4bXNJS3BhVjhoMEFHRTA1c28wZkF3d2ZUdy41NkI0NEY2RDEwNTU3Q0M2",
            )

        # test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_1)

            res_by_playlist = self.api.get_playlist_items(
                playlist_id="PLOU2XLYxmsIKpaV8h0AGE05so0fAwwfTw",
                parts="id,snippet",
                limit=10,
                count=5,
            )
            self.assertEqual(res_by_playlist.kind, "youtube#playlistItemListResponse")
            self.assertEqual(res_by_playlist.pageInfo.totalResults, 13)
            self.assertEqual(len(res_by_playlist.items), 5)
            self.assertEqual(
                res_by_playlist.items[0].id,
                "UExPVTJYTFl4bXNJS3BhVjhoMEFHRTA1c28wZkF3d2ZUdy41NkI0NEY2RDEwNTU3Q0M2",
            )

        # test get all items
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_2)

            res_by_playlist = self.api.get_playlist_items(
                playlist_id="PLOU2XLYxmsIKpaV8h0AGE05so0fAwwfTw",
                parts="id,snippet",
                count=None,
            )
            self.assertEqual(res_by_playlist.pageInfo.totalResults, 13)
            self.assertEqual(len(res_by_playlist.items), 13)

        # test filter
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_FILTER)

            res_by_filter = self.api.get_playlist_items(
                playlist_id="PLOU2XLYxmsIKpaV8h0AGE05so0fAwwfTw",
                parts=("id", "snippet"),
                video_id="VCv-KKIkLns",
                return_json=True,
            )

            self.assertEqual(res_by_filter["pageInfo"]["totalResults"], 1)
            self.assertEqual(
                res_by_filter["items"][0]["snippet"]["resourceId"]["videoId"],
                "VCv-KKIkLns",
            )

        # test use page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLIST_ITEM_PAGED_2)

            res_by_playlist = self.api.get_playlist_items(
                playlist_id="PLOU2XLYxmsIKpaV8h0AGE05so0fAwwfTw",
                parts="id,snippet",
                page_token="CAoQAA",
                count=3,
            )
            self.assertEqual(len(res_by_playlist.items), 3)



import json
import unittest

import responses

import pyyoutube


class ApiVideoTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/videos/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/videos"

    with open(BASE_PATH + "videos_info_single.json", "rb") as f:
        VIDEOS_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_info_multi.json", "rb") as f:
        VIDEOS_INFO_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_chart_paged_1.json", "rb") as f:
        VIDEOS_CHART_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_chart_paged_2.json", "rb") as f:
        VIDEOS_CHART_PAGED_2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_myrating_paged_1.json", "rb") as f:
        VIDEOS_MYRATING_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "videos_myrating_paged_2.json", "rb") as f:
        VIDEOS_MYRATING_PAGED_2 = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")
        self.api_with_token = pyyoutube.Api(access_token="token")

    def testGetVideoById(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_video_by_id(video_id="id", parts="id,not_part")

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_INFO_SINGLE)
            m.add("GET", self.BASE_URL, json=self.VIDEOS_INFO_MULTI)

            res_by_single_id = self.api.get_video_by_id(
                video_id="D-lhorsDlUQ",
                parts="id,snippet,player",
                max_height=480,
                max_width=270,
                return_json=True,
            )
            self.assertEqual(res_by_single_id["kind"], "youtube#videoListResponse")
            self.assertEqual(res_by_single_id["pageInfo"]["totalResults"], 1)
            video = res_by_single_id["items"][0]
            self.assertEqual(video["id"], "D-lhorsDlUQ")
            self.assertEqual(
                video["player"]["embedHtml"],
                (
                    '\u003ciframe width="480" height="270" src="//www.youtube.com/embed/D-lhorsDlUQ" '
                    'frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; '
                    'picture-in-picture" allowfullscreen\u003e\u003c/iframe\u003e'
                ),
            )

            res_by_multi_id = self.api.get_video_by_id(
                video_id=["D-lhorsDlUQ", "ovdbrdCIP7U"]
            )
            self.assertEqual(res_by_multi_id.pageInfo.totalResults, 2)
            self.assertEqual(len(res_by_multi_id.items), 2)
            self.assertEqual(res_by_multi_id.items[0].id, "D-lhorsDlUQ")

    def testGetVideoByChart(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_videos_by_chart(chart="mostPopular", parts="id,not_part")

        # test paged
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_2)

            res_by_chart = self.api.get_videos_by_chart(
                chart="mostPopular",
                region_code="US",
                category_id="0",
                max_height=480,
                max_width=270,
                count=20,
                limit=5,
                return_json=True,
            )
            self.assertEqual(res_by_chart["kind"], "youtube#videoListResponse")
            self.assertEqual(res_by_chart["pageInfo"]["totalResults"], 8)
            self.assertEqual(len(res_by_chart["items"]), 8)
            self.assertEqual(res_by_chart["items"][0]["id"], "hDeuSfo_Ys0")

        # test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_1)

            res_by_chart = self.api.get_videos_by_chart(chart="mostPopular", count=3)
            self.assertEqual(res_by_chart.pageInfo.totalResults, 8)
            self.assertEqual(len(res_by_chart.items), 3)
            self.assertEqual(res_by_chart.items[0].id, "hDeuSfo_Ys0")

        # test get all items
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_2)

            res_by_chart = self.api.get_videos_by_chart(chart="mostPopular", count=None)
            self.assertEqual(res_by_chart.pageInfo.totalResults, 8)

        # test use page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_CHART_PAGED_2)

            res_by_chart = self.api.get_videos_by_chart(
                chart="mostPopular", count=None, page_token="CAUQAA"
            )
            self.assertEqual(len(res_by_chart.items), 3)

    def testGetVideoByMyRating(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api_with_token.get_videos_by_myrating(
                rating="like", parts="id,not_part"
            )

        # test need authorization
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_videos_by_myrating(rating="like", parts="id,not_part")

        # test paged
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_2)

            res_by_my_rating = self.api_with_token.get_videos_by_myrating(
                rating="like",
                parts=("id", "snippet", "player"),
                max_height=480,
                max_width=270,
                count=10,
                limit=2,
                return_json=True,
            )
            self.assertEqual(res_by_my_rating["kind"], "youtube#videoListResponse")
            self.assertEqual(res_by_my_rating["pageInfo"]["totalResults"], 3)
            self.assertEqual(len(res_by_my_rating["items"]), 3)
            self.assertEqual(res_by_my_rating["items"][0]["id"], "P4IfFLAX9hY")

        # test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_1)
            res_by_my_rating = self.api_with_token.get_videos_by_myrating(
                rating="like",
                parts=("id", "snippet", "player"),
                count=1,
                limit=2,
            )
            self.assertEqual(res_by_my_rating.pageInfo.totalResults, 3)
            self.assertEqual(len(res_by_my_rating.items), 1)
            self.assertEqual(res_by_my_rating.items[0].id, "P4IfFLAX9hY")

        # test get all items
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_2)
            res_by_my_rating = self.api_with_token.get_videos_by_myrating(
                rating="like",
                parts=("id", "snippet", "player"),
                count=None,
            )
            self.assertEqual(res_by_my_rating.pageInfo.totalResults, 3)

        # test use page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEOS_MYRATING_PAGED_2)

            res_by_my_rating = self.api_with_token.get_videos_by_myrating(
                rating="like",
                parts=("id", "snippet", "player"),
                count=None,
                page_token="CAIQAA",
            )
            self.assertEqual(len(res_by_my_rating.items), 1)



from dataclasses import dataclass
from typing import Optional, Union

from requests import Response

__all__ = ["ErrorCode", "ErrorMessage", "PyYouTubeException"]


class ErrorCode:
    HTTP_ERROR = 10000
    MISSING_PARAMS = 10001
    INVALID_PARAMS = 10002
    NEED_AUTHORIZATION = 10003
    AUTHORIZE_URL_FIRST = 10004


@dataclass
class ErrorMessage:
    status_code: Optional[int] = None
    message: Optional[str] = None


class PyYouTubeException(Exception):
    """
    This is a return demo:
    {'error': {'errors': [{'domain': 'youtube.parameter',
    'reason': 'missingRequiredParameter',
    'message': 'No filter selected. Expected one of: forUsername, managedByMe, categoryId, mine, mySubscribers, id, idParam',
    'locationType': 'parameter',
    'location': ''}],
    'code': 400,
    'message': 'No filter selected. Expected one of: forUsername, managedByMe, categoryId, mine, mySubscribers, id, idParam'}}
    """

    def __init__(self, response: Optional[Union[ErrorMessage, Response]]):
        self.status_code: Optional[int] = None
        self.error_type: Optional[str] = None
        self.message: Optional[str] = None
        self.response: Optional[Union[ErrorMessage, Response]] = response
        self.error_handler()

    def error_handler(self):
        """
        Error has two big type(but not the error type.): This module's error, Api return error.
        So This will change two error to one format
        """
        if isinstance(self.response, ErrorMessage):
            self.status_code = self.response.status_code
            self.message = self.response.message
            self.error_type = "PyYouTubeException"
        elif isinstance(self.response, Response):
            res_data = self.response.json()
            if "error" in res_data:
                self.status_code = res_data["error"]["code"]
                self.message = res_data["error"]["message"]
                self.error_type = "YouTubeException"

    def __repr__(self):
        return (
            f"{self.error_type}(status_code={self.status_code},message={self.message})"
        )

    def __str__(self):
        return self.__repr__()



import json
import unittest

import responses

import pyyoutube


class ApiCommentTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/comments/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/comments"

    with open(BASE_PATH + "comments_single.json", "rb") as f:
        COMMENTS_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comments_multi.json", "rb") as f:
        COMMENTS_INFO_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comments_by_parent_paged_1.json", "rb") as f:
        COMMENTS_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comments_by_parent_paged_2.json", "rb") as f:
        COMMENTS_PAGED_2 = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testGetCommentById(self) -> None:
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_comment_by_id(comment_id="id", parts="id,not_part")

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENTS_INFO_SINGLE)
            m.add("GET", self.BASE_URL, json=self.COMMENTS_INFO_MULTI)

            res_by_single = self.api.get_comment_by_id(
                comment_id="UgyUBI0HsgL9emxcZpR4AaABAg",
                parts=["id", "snippet"],
                return_json=True,
            )
            self.assertEqual(res_by_single["kind"], "youtube#commentListResponse")
            self.assertEqual(len(res_by_single["items"]), 1)
            self.assertEqual(
                res_by_single["items"][0]["id"], "UgyUBI0HsgL9emxcZpR4AaABAg"
            )

            res_by_multi = self.api.get_comment_by_id(
                comment_id=["UgyUBI0HsgL9emxcZpR4AaABAg", "Ugzi3lkqDPfIOirGFLh4AaABAg"],
                parts=("id", "snippet"),
            )
            self.assertEqual(len(res_by_multi.items), 2)
            self.assertEqual(res_by_multi.items[1].id, "Ugzi3lkqDPfIOirGFLh4AaABAg")

    def testGetCommentsByParentId(self) -> None:
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_comments(parent_id="id", parts="id,not_part")

        # test paged
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_2)

            res_by_parent = self.api.get_comments(
                parent_id="Ugw5zYU6n9pmIgAZWvN4AaABAg",
                parts="id,snippet",
                limit=2,
            )
            self.assertEqual(res_by_parent.kind, "youtube#commentListResponse")
            self.assertEqual(len(res_by_parent.items), 3)
            self.assertEqual(
                res_by_parent.items[0].id,
                "Ugw5zYU6n9pmIgAZWvN4AaABAg.91zT3cYb5B291za6voUoRh",
            )

        # test count
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_1)

            res_by_parent = self.api.get_comments(
                parent_id="Ugw5zYU6n9pmIgAZWvN4AaABAg",
                parts="id,snippet",
                count=2,
                limit=2,
                return_json=True,
            )
            self.assertEqual(len(res_by_parent["items"]), 2)
            self.assertEqual(
                res_by_parent["items"][0]["id"],
                "Ugw5zYU6n9pmIgAZWvN4AaABAg.91zT3cYb5B291za6voUoRh",
            )

        # test get all comments
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_2)
            res_by_parent = self.api.get_comments(
                parent_id="Ugw5zYU6n9pmIgAZWvN4AaABAg", parts="id,snippet", count=None
            )
            self.assertEqual(len(res_by_parent.items), 3)

        # test use page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENTS_PAGED_2)
            res_by_parent = self.api.get_comments(
                parent_id="Ugw5zYU6n9pmIgAZWvN4AaABAg",
                parts="id,snippet",
                count=None,
                page_token="R0FJeVZnbzBJTl9zNXRxNXlPWUNNaWtRQUJpQ3RNeW4wcFBtQWlBQktBTXdDam9XT1RGNlZETmpXV0kxUWpJNU1YcGhOV1ZLZUhwek1SSWVDQVVTR2xWbmR6VjZXVlUyYmpsd2JVbG5RVnBYZGs0MFFXRkJRa0ZuT2lBSUFSSWNOVHBWWjNjMWVsbFZObTQ1Y0cxSlowRmFWM1pPTkVGaFFVSkJadw==",
            )
            self.assertEqual(len(res_by_parent.items), 1)



import json
import unittest

import pyyoutube.models as models


class PlaylistItemModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/playlist_items/"

    with open(BASE_PATH + "playlist_item_content_details.json", "rb") as f:
        CONTENT_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_item_snippet.json", "rb") as f:
        SNIPPET_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_item_status.json", "rb") as f:
        STATUS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_item_info.json", "rb") as f:
        PLAYLIST_ITEM_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlist_item_api_response.json", "rb") as f:
        PLAYLIST_LIST_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testPlaylistItemContentDetails(self) -> None:
        m = models.PlaylistItemContentDetails.from_dict(self.CONTENT_DETAILS_INFO)

        self.assertEqual(m.videoId, "D-lhorsDlUQ")
        self.assertEqual(
            m.string_to_datetime(m.videoPublishedAt).isoformat(),
            "2019-03-21T20:37:49+00:00",
        )

    def testPlaylistItemSnippet(self) -> None:
        m = models.PlaylistItemSnippet.from_dict(self.SNIPPET_INFO)

        self.assertEqual(
            m.string_to_datetime(m.publishedAt).isoformat(), "2019-05-16T18:46:20+00:00"
        )
        self.assertEqual(m.title, "What are Actions on Google (Assistant on Air)")
        self.assertEqual(
            m.thumbnails.default.url, "https://i.ytimg.com/vi/D-lhorsDlUQ/default.jpg"
        )
        self.assertEqual(m.resourceId.videoId, "D-lhorsDlUQ")

    def testPlaylistItemStatus(self) -> None:
        m = models.PlaylistItemStatus.from_dict(self.STATUS_INFO)

        self.assertEqual(m.privacyStatus, "public")

    def testPlaylistItem(self) -> None:
        m = models.PlaylistItem.from_dict(self.PLAYLIST_ITEM_INFO)

        self.assertEqual(
            m.id, "UExPVTJYTFl4bXNJSnB1ZmVNSG5jblF2Rk9lMEszTWhWcC41NkI0NEY2RDEwNTU3Q0M2"
        )
        self.assertEqual(m.snippet.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw")
        self.assertEqual(m.snippet.resourceId.videoId, "D-lhorsDlUQ")
        self.assertEqual(m.contentDetails.videoId, "D-lhorsDlUQ")
        self.assertEqual(m.status.privacyStatus, "public")
        self.assertEqual(m.snippet.videoOwnerChannelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw")

    def testPlaylistItemListResponse(self) -> None:
        m = models.PlaylistItemListResponse.from_dict(self.PLAYLIST_LIST_RESPONSE)

        self.assertEqual(m.kind, "youtube#playlistItemListResponse")
        self.assertEqual(m.pageInfo.totalResults, 3)
        self.assertEqual(len(m.items), 3)
        self.assertEqual(
            m.items[0].id,
            "UExPVTJYTFl4bXNJSlhzSDJodEcxZzBOVWpIR3E2MlE3aS41NkI0NEY2RDEwNTU3Q0M2",
        )



import json
import unittest

import responses
import pyyoutube


class ApiMembersTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/members/"
    MEMBERS_URL = "https://www.googleapis.com/youtube/v3/members"
    MEMBERSHIP_LEVEL_URL = "https://www.googleapis.com/youtube/v3/membershipsLevels"

    with open(BASE_PATH + "members_data.json", "rb") as f:
        MEMBERS_RES = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "membership_levels.json", "rb") as f:
        MEMBERSHIP_LEVEL_RES = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(access_token="Authorize token")

    def testGetMembers(self) -> None:
        with responses.RequestsMock() as m:
            m.add("GET", self.MEMBERS_URL, json=self.MEMBERS_RES)

            members = self.api.get_members(parts=["id", "snippet"])
            self.assertEqual(members.kind, "youtube#memberListResponse")
            self.assertEqual(len(members.items), 2)

            members_json = self.api.get_members(
                page_token="token",
                count=None,
                has_access_to_level="high",
                filter_by_member_channel_id="id",
                return_json=True,
            )
            self.assertEqual(len(members_json["items"]), 2)

    def testGetMembershipLevels(self) -> None:
        with responses.RequestsMock() as m:
            m.add("GET", self.MEMBERSHIP_LEVEL_URL, json=self.MEMBERSHIP_LEVEL_RES)

            membership_levels = self.api.get_membership_levels(parts=["id", "snippet"])
            self.assertEqual(
                membership_levels.kind, "youtube#membershipsLevelListResponse"
            )
            self.assertEqual(len(membership_levels.items), 2)

            membership_levels_json = self.api.get_membership_levels(return_json=True)
            self.assertEqual(len(membership_levels_json["items"]), 2)



"""
    some constants for YouTube
"""

CHANNEL_RESOURCE_PROPERTIES = {
    "id",
    "brandingSettings",
    "contentDetails",
    "localizations",
    "snippet",
    "statistics",
    "status",
    "topicDetails",
}

CHANNEL_SECTIONS_PROPERTIES = {
    "id",
    "contentDetails",
    "localizations",
    "snippet",
    "targeting",
}

PLAYLIST_RESOURCE_PROPERTIES = {
    "id",
    "contentDetails",
    "localizations",
    "player",
    "snippet",
    "status",
}

PLAYLIST_ITEM_RESOURCE_PROPERTIES = {"id", "contentDetails", "snippet", "status"}

VIDEO_RESOURCE_PROPERTIES = {
    "id",
    "contentDetails",
    "player",
    "snippet",
    "statistics",
    "status",
    "topicDetails",
    "liveStreamingDetails",
}

COMMENT_THREAD_RESOURCE_PROPERTIES = {"id", "replies", "snippet"}

COMMENT_RESOURCE_PROPERTIES = {"id", "snippet"}

VIDEO_CATEGORY_RESOURCE_PROPERTIES = {"id", "snippet"}

GUIDE_CATEGORY_RESOURCE_PROPERTIES = {"id", "snippet"}

SEARCH_RESOURCE_PROPERTIES = {"id", "snippet"}

SUBSCRIPTION_RESOURCE_PROPERTIES = {
    "id",
    "snippet",
    "contentDetails",
    "subscriberSnippet",
}

ACTIVITIES_RESOURCE_PROPERTIES = {
    "id",
    "snippet",
    "contentDetails",
}

CAPTIONS_RESOURCE_PROPERTIES = {
    "id",
    "snippet",
}

I18N_REGION_PROPERTIES = {
    "id",
    "snippet",
}

I18N_LANGUAGE_PROPERTIES = {
    "id",
    "snippet",
}

VIDEO_ABUSE_REPORT_REASON_PROPERTIES = {
    "id",
    "snippet",
}

MEMBER_PROPERTIES = {
    "id",
    "snippet",
}

MEMBERSHIP_LEVEL_PROPERTIES = {
    "id",
    "snippet",
}

RESOURCE_PARTS_MAPPING = {
    "channels": CHANNEL_RESOURCE_PROPERTIES,
    "channelSections": CHANNEL_SECTIONS_PROPERTIES,
    "playlists": PLAYLIST_RESOURCE_PROPERTIES,
    "playlistItems": PLAYLIST_ITEM_RESOURCE_PROPERTIES,
    "videos": VIDEO_RESOURCE_PROPERTIES,
    "commentThreads": COMMENT_THREAD_RESOURCE_PROPERTIES,
    "comments": COMMENT_RESOURCE_PROPERTIES,
    "videoCategories": VIDEO_CATEGORY_RESOURCE_PROPERTIES,
    "guideCategories": GUIDE_CATEGORY_RESOURCE_PROPERTIES,
    "search": SEARCH_RESOURCE_PROPERTIES,
    "subscriptions": SUBSCRIPTION_RESOURCE_PROPERTIES,
    "activities": ACTIVITIES_RESOURCE_PROPERTIES,
    "captions": CAPTIONS_RESOURCE_PROPERTIES,
    "i18nRegions": I18N_REGION_PROPERTIES,
    "i18nLanguages": I18N_LANGUAGE_PROPERTIES,
    "videoAbuseReportReasons": VIDEO_ABUSE_REPORT_REASON_PROPERTIES,
    "members": MEMBER_PROPERTIES,
    "membershipsLevels": MEMBERSHIP_LEVEL_PROPERTIES,
}

TOPICS = {
    # Music topics
    "/m/04rlf": "Music (parent topic)",
    "/m/02mscn": "Christian music",
    "/m/0ggq0m": "Classical music",
    "/m/01lyv": "Country",
    "/m/02lkt": "Electronic music",
    "/m/0glt670": "Hip hop music",
    "/m/05rwpb": "Independent music",
    "/m/03_d0": "Jazz",
    "/m/028sqc": "Music of Asia",
    "/m/0g293": "Music of Latin America",
    "/m/064t9": "Pop music",
    "/m/06cqb": "Reggae",
    "/m/06j6l": "Rhythm and blues",
    "/m/06by7": "Rock music",
    "/m/0gywn": "Soul music",
    # Gaming topics
    "/m/0bzvm2": "Gaming (parent topic)",
    "/m/025zzc": "Action game",
    "/m/02ntfj": "Action-adventure game",
    "/m/0b1vjn": "Casual game",
    "/m/02hygl": "Music video game",
    "/m/04q1x3q": "Puzzle video game",
    "/m/01sjng": "Racing video game",
    "/m/0403l3g": "Role-playing video game",
    "/m/021bp2": "Simulation video game",
    "/m/022dc6": "Sports game",
    "/m/03hf_rm": "Strategy video game",
    # Sports topics
    "/m/06ntj": "Sports (parent topic)",
    "/m/0jm_": "American football",
    "/m/018jz": "Baseball",
    "/m/018w8": "Basketball",
    "/m/01cgz": "Boxing",
    "/m/09xp_": "Cricket",
    "/m/02vx4": "Football",
    "/m/037hz": "Golf",
    "/m/03tmr": "Ice hockey",
    "/m/01h7lh": "Mixed martial arts",
    "/m/0410tth": "Motorsport",
    "/m/07bs0": "Tennis",
    "/m/07_53": "Volleyball",
    # Entertainment topics
    "/m/02jjt": "Entertainment (parent topic)",
    "/m/09kqc": "Humor",
    "/m/02vxn": "Movies",
    "/m/05qjc": "Performing arts",
    "/m/066wd": "Professional wrestling",
    "/m/0f2f9": "TV shows",
    # Lifestyle topics
    "/m/019_rr": "Lifestyle (parent topic)",
    "/m/032tl": "Fashion",
    "/m/027x7n": "Fitness",
    "/m/02wbm": "Food",
    "/m/03glg": "Hobby",
    "/m/068hy": "Pets",
    "/m/041xxh": "Physical attractiveness [Beauty]",
    "/m/07c1v": "Technology",
    "/m/07bxq": "Tourism",
    "/m/07yv9": "Vehicles",
    # Society topics
    "/m/098wr": "Society (parent topic)",
    "/m/09s1f": "Business",
    "/m/0kt51": "Health",
    "/m/01h6rj": "Military",
    "/m/05qt0": "Politics",
    "/m/06bvp": "Religion",
    # Other topics
    "/m/01k8wb": "Knowledge",
}



import json
import unittest
import pyyoutube.models as models


class AuthModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/users/"
    with open(BASE_PATH + "access_token.json", "rb") as f:
        ACCESS_TOKEN_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "user_profile.json", "rb") as f:
        USER_PROFILE_INFO = json.loads(f.read().decode("utf-8"))

    def testAccessToken(self) -> None:
        m = models.AccessToken.from_dict(self.ACCESS_TOKEN_INFO)

        self.assertEqual(m.access_token, "access_token")

    def testUserProfile(self) -> None:
        m = models.UserProfile.from_dict(self.USER_PROFILE_INFO)

        self.assertEqual(m.id, "12345678910")

        origin_data = json.dumps(self.USER_PROFILE_INFO, sort_keys=True)
        d = m.to_json(sort_keys=True, allow_nan=False)
        self.assertEqual(origin_data, d)



import json
import unittest

import pyyoutube
import responses


class ApiChannelSectionTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/channel_sections/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/channelSections"

    with open(BASE_PATH + "channel_sections_by_id.json", "rb") as f:
        CHANNEL_SECTIONS_BY_ID = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_sections_by_ids.json", "rb") as f:
        CHANNEL_SECTIONS_BY_IDS = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_sections_by_channel.json", "rb") as f:
        CHANNEL_SECTIONS_BY_CHANNEL = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testGetChannelSectionsById(self) -> None:
        section_id = "UCa-vrCLQHviTOVnEKDOdetQ.nGzAI5pLbMY"
        section_ids = [
            "UC_x5XG1OV2P6uZZ5FSM9Ttw.npYvuMz0_es",
            "UC_x5XG1OV2P6uZZ5FSM9Ttw.9_wU0qhEPR8",
        ]

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CHANNEL_SECTIONS_BY_ID)
            m.add("GET", self.BASE_URL, json=self.CHANNEL_SECTIONS_BY_IDS)

            section_res = self.api.get_channel_sections_by_id(
                section_id=section_id,
            )
            self.assertEqual(section_res.kind, "youtube#channelSectionListResponse")
            self.assertEqual(len(section_res.items), 1)
            self.assertEqual(section_res.items[0].id, section_id)

            section_multi_res = self.api.get_channel_sections_by_id(
                section_id=section_ids, parts=["id", "snippet"], return_json=True
            )

            self.assertEqual(len(section_multi_res["items"]), 2)
            self.assertIn(section_multi_res["items"][1]["id"], section_ids)

    def testGetChannelSectionsByChannel(self) -> None:
        channel_id = "UCa-vrCLQHviTOVnEKDOdetQ"

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CHANNEL_SECTIONS_BY_CHANNEL)

            section_by_channel = self.api.get_channel_sections_by_channel(
                channel_id=channel_id,
            )

            self.assertEqual(len(section_by_channel.items), 3)
            self.assertEqual(
                section_by_channel.items[0].id, "UCa-vrCLQHviTOVnEKDOdetQ.jNQXAC9IVRw"
            )

            section_by_me = self.api.get_channel_sections_by_channel(
                mine=True,
                return_json=True,
            )

            self.assertEqual(
                section_by_me["items"][2]["id"], "UCa-vrCLQHviTOVnEKDOdetQ.nGzAI5pLbMY"
            )



import json
import unittest

import responses

import pyyoutube


class ApiCaptionsTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/captions/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/captions"

    with open(BASE_PATH + "captions_by_video.json", "rb") as f:
        CAPTIONS_BY_VIDEO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "captions_filter_by_id.json", "rb") as f:
        CAPTIONS_FILTER_ID = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api_with_access_token = pyyoutube.Api(access_token="token")

    def testGetCaptionByVideo(self) -> None:
        video_id = "oHR3wURdJ94"

        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api_with_access_token.get_captions_by_video(
                video_id=video_id,
                parts="id,not_part",
            )

        # test by video
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CAPTIONS_BY_VIDEO)

            res = self.api_with_access_token.get_captions_by_video(
                video_id=video_id,
                parts="id,snippet",
                return_json=True,
            )
            self.assertEqual(len(res["items"]), 2)
            self.assertEqual(res["items"][0]["snippet"]["videoId"], video_id)

        # test filter id
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CAPTIONS_FILTER_ID)

            res = self.api_with_access_token.get_captions_by_video(
                video_id=video_id,
                parts=["id", "snippet"],
                caption_id="SwPOvp0r7kd9ttt_XhcHdZthMwXG7Z0I",
            )

            self.assertEqual(len(res.items), 1)
            self.assertEqual(res.items[0].snippet.videoId, video_id)



"""
    These are comment and comment threads related models.
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .mixins import DatetimeTimeMixin
from .common import BaseApiResponse, BaseResource


@dataclass
class CommentSnippetAuthorChannelId(BaseModel):
    """
    A class representing comment's snippet authorChannelId info.

    Refer: https://developers.google.com/youtube/v3/docs/comments#snippet.authorChannelId
    """

    value: Optional[str] = field(default=None)


@dataclass
class CommentSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing comment's snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/comments#snippet
    """

    authorDisplayName: Optional[str] = field(default=None)
    authorProfileImageUrl: Optional[str] = field(default=None, repr=False)
    authorChannelUrl: Optional[str] = field(default=None, repr=False)
    authorChannelId: Optional[CommentSnippetAuthorChannelId] = field(
        default=None, repr=False
    )
    channelId: Optional[str] = field(default=None, repr=False)
    videoId: Optional[str] = field(default=None, repr=False)
    textDisplay: Optional[str] = field(default=None, repr=False)
    textOriginal: Optional[str] = field(default=None, repr=False)
    parentId: Optional[str] = field(default=None, repr=False)
    canRate: Optional[bool] = field(default=None, repr=False)
    viewerRating: Optional[str] = field(default=None, repr=False)
    likeCount: Optional[int] = field(default=None)
    moderationStatus: Optional[str] = field(default=None, repr=False)
    publishedAt: Optional[str] = field(default=None, repr=False)
    updatedAt: Optional[str] = field(default=None, repr=False)


@dataclass
class Comment(BaseResource):
    """
    A class representing comment info.

    Refer: https://developers.google.com/youtube/v3/docs/comments
    """

    snippet: Optional[CommentSnippet] = field(default=None)


@dataclass
class CommentListResponse(BaseApiResponse):
    """
    A class representing the comment's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/comments/list#response_1
    """

    items: Optional[List[Comment]] = field(default=None, repr=False)


@dataclass
class CommentThreadSnippet(BaseModel):
    """
    A class representing comment tread snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads#snippet
    """

    channelId: Optional[str] = field(default=None)
    videoId: Optional[str] = field(default=None)
    topLevelComment: Optional[Comment] = field(default=None, repr=False)
    canReply: Optional[bool] = field(default=None, repr=False)
    totalReplyCount: Optional[int] = field(default=None, repr=False)
    isPublic: Optional[bool] = field(default=None, repr=False)


@dataclass
class CommentThreadReplies(BaseModel):
    """
    A class representing comment tread replies info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads#replies
    """

    comments: Optional[List[Comment]] = field(default=None, repr=False)


@dataclass
class CommentThread(BaseResource):
    """
    A class representing comment thread info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads
    """

    snippet: Optional[CommentThreadSnippet] = field(default=None, repr=False)
    replies: Optional[CommentThreadReplies] = field(default=None, repr=False)


@dataclass
class CommentThreadListResponse(BaseApiResponse):
    """
    A class representing the comment thread's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/commentThreads/list#response_1
    """

    items: Optional[List[CommentThread]] = field(default=None, repr=False)



import json
import unittest

from requests import HTTPError

import pyyoutube
import responses


class ApiChannelTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/channels"

    with open(BASE_PATH + "channel_info_single.json", "rb") as f:
        CHANNELS_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_info_multi.json", "rb") as f:
        CHANNELS_INFO_MULTI = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testSendRequest(self) -> None:
        with self.assertRaises(pyyoutube.PyYouTubeException):
            api = pyyoutube.Api(client_id="id", client_secret="secret")
            api._request("channels", post_args={"a": "a"})
        with responses.RequestsMock() as m:
            m.add("POST", self.BASE_URL, json={})
            api = pyyoutube.Api(access_token="access token")
            res = api._request("channels", post_args={"a": "a"})
            self.assertTrue(res)

        with self.assertRaises(pyyoutube.PyYouTubeException):
            with responses.RequestsMock() as m:
                m.add("GET", self.BASE_URL, body=HTTPError("Exception"))
                self.api.get_channel_info(channel_id="channel_id", parts="id,snippet")

    # TODO need to separate.
    def testParseResponse(self) -> None:
        with open("testdata/error_response.json", "rb") as f:
            error_response = json.loads(f.read().decode("utf-8"))

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=error_response, status=400)

            with self.assertRaises(pyyoutube.PyYouTubeException):
                self.api.get_channel_info(
                    channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw", parts="id,snippet,statistics"
                )

    def testGetChannelInfo(self) -> None:
        # test params checker
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_channel_info(parts="id,invideoPromotion")
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_channel_info()

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CHANNELS_INFO_SINGLE)

            res_by_channel_id = self.api.get_channel_info(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw", parts="id,snippet,statistics"
            )
            self.assertEqual(res_by_channel_id.items[0].id, "UC_x5XG1OV2P6uZZ5FSM9Ttw")

            res_by_channel_name = self.api.get_channel_info(
                for_username="GoogleDevelopers", return_json=True
            )
            self.assertEqual(
                res_by_channel_name["items"][0]["id"], "UC_x5XG1OV2P6uZZ5FSM9Ttw"
            )

            res_by_mine = self.api.get_channel_info(mine=True)
            self.assertEqual(res_by_mine.items[0].id, "UC_x5XG1OV2P6uZZ5FSM9Ttw")

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.CHANNELS_INFO_MULTI)

            res_by_channel_id_list = self.api.get_channel_info(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw,UCK8sQmJBp8GCxrOtXWBpyEA",
                parts="id,snippet",
            )
            self.assertEqual(len(res_by_channel_id_list.items), 2)
            self.assertEqual(
                res_by_channel_id_list.items[1].id, "UCK8sQmJBp8GCxrOtXWBpyEA"
            )



"""
    These are channel related models.
"""
from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .common import BaseResource, BaseTopicDetails, Thumbnails, BaseApiResponse
from .mixins import DatetimeTimeMixin


@dataclass
class ChannelBrandingChannel(BaseModel):
    """
    A class representing the channel branding setting's channel info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#brandingSettings.channel
    """

    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    keywords: Optional[str] = field(default=None, repr=False)
    trackingAnalyticsAccountId: Optional[str] = field(default=None, repr=False)
    moderateComments: Optional[bool] = field(default=None, repr=False)
    unsubscribedTrailer: Optional[str] = field(default=None, repr=False)
    defaultLanguage: Optional[str] = field(default=None, repr=False)
    country: Optional[str] = field(default=None, repr=False)


@dataclass
class ChannelBrandingImage(BaseModel):
    """
    A class representing the channel branding setting's image info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#brandingSettings.image
    """

    bannerImageUrl: Optional[str] = field(default=None)
    bannerMobileImageUrl: Optional[str] = field(default=None, repr=False)
    watchIconImageUrl: Optional[str] = field(default=None, repr=False)
    trackingImageUrl: Optional[str] = field(default=None, repr=False)
    bannerTabletLowImageUrl: Optional[str] = field(default=None, repr=False)
    bannerTabletImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTabletHdImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTabletExtraHdImageUrl: Optional[str] = field(default=None, repr=None)
    bannerMobileLowImageUrl: Optional[str] = field(default=None, repr=None)
    bannerMobileMediumHdImageUrl: Optional[str] = field(default=None, repr=None)
    bannerMobileHdImageUrl: Optional[str] = field(default=None, repr=None)
    bannerMobileExtraHdImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTvImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTvLowImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTvMediumImageUrl: Optional[str] = field(default=None, repr=None)
    bannerTvHighImageUrl: Optional[str] = field(default=None, repr=None)
    bannerExternalUrl: Optional[str] = field(default=None, repr=None)


@dataclass
class ChannelBrandingHint(BaseModel):
    """
    A class representing the channel branding setting's hint info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#brandingSettings.hints
    """

    property: Optional[str] = field(default=None)
    value: Optional[str] = field(default=None)


@dataclass
class ChannelBrandingSetting(BaseModel):
    """
    A class representing the channel branding settings info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#brandingSettings
    """

    channel: Optional[ChannelBrandingChannel] = field(default=None)


@dataclass
class RelatedPlaylists(BaseModel):
    """
    A class representing the channel's related playlist info

    Refer: https://developers.google.com/youtube/v3/docs/channels#contentDetails.relatedPlaylists
    """

    likes: Optional[str] = field(default=None, repr=False)
    uploads: Optional[str] = field(default=None)


@dataclass
class ChannelContentDetails(BaseModel):
    """
    A class representing the channel's content info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#contentDetails
    """

    relatedPlaylists: Optional[RelatedPlaylists] = field(default=None)


@dataclass
class ChannelTopicDetails(BaseTopicDetails):
    """
    A class representing the channel's topic detail info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#topicDetails
    """

    # Important:
    # topicIds maybe has deprecated.
    # see more: https://developers.google.com/youtube/v3/revision_history#november-10-2016
    topicIds: Optional[List[str]] = field(default=None, repr=False)
    topicCategories: Optional[List[str]] = field(default=None)


@dataclass
class Localized(BaseModel):
    """
    A class representing the channel snippet localized info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#snippet.localized
    """

    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None, repr=False)


@dataclass
class ChannelSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing the channel snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#snippet
    """

    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    customUrl: Optional[str] = field(default=None, repr=False)
    publishedAt: Optional[str] = field(default=None, repr=False)
    thumbnails: Optional[Thumbnails] = field(default=None, repr=False)
    defaultLanguage: Optional[str] = field(default=None, repr=False)
    localized: Optional[Localized] = field(default=None, repr=False)
    country: Optional[str] = field(default=None, repr=False)


@dataclass
class ChannelStatistics(BaseModel):
    """
    A class representing the Channel's statistics info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#statistics
    """

    viewCount: Optional[int] = field(default=None)
    subscriberCount: Optional[int] = field(default=None)
    hiddenSubscriberCount: Optional[bool] = field(default=None, repr=False)
    videoCount: Optional[int] = field(default=None, repr=False)


@dataclass
class ChannelStatus(BaseModel):
    """
    A class representing the channel's status info.

    Refer: https://developers.google.com/youtube/v3/docs/channels#status
    """

    privacyStatus: Optional[str] = field(default=None)
    isLinked: Optional[bool] = field(default=None, repr=False)
    longUploadsStatus: Optional[str] = field(default=None, repr=False)


@dataclass
class Channel(BaseResource):
    """
    A class representing the channel's info.

    Refer: https://developers.google.com/youtube/v3/docs/channels
    """

    snippet: Optional[ChannelSnippet] = field(default=None, repr=False)
    contentDetails: Optional[ChannelContentDetails] = field(default=None, repr=False)
    statistics: Optional[ChannelStatistics] = field(default=None, repr=False)
    topicDetails: Optional[ChannelTopicDetails] = field(default=None, repr=False)
    status: Optional[ChannelStatus] = field(default=None, repr=False)
    brandingSettings: Optional[ChannelBrandingSetting] = field(default=None, repr=False)


@dataclass
class ChannelListResponse(BaseApiResponse):
    """
    A class representing the channel's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/channels/list#response_1
    """

    items: Optional[List[Channel]] = field(default=None, repr=False)



import json

import unittest

import pyyoutube.models as models


class I18nModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/i18ns/"

    with open(BASE_PATH + "region_info.json", "rb") as f:
        REGION_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "region_res.json", "rb") as f:
        REGION_RES = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "language_info.json", "rb") as f:
        LANGUAGE_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "language_res.json", "rb") as f:
        LANGUAGE_RES = json.loads(f.read().decode("utf-8"))

    def testI18nRegion(self) -> None:
        m = models.I18nRegion.from_dict(self.REGION_INFO)

        self.assertEqual(m.id, "DZ")
        self.assertEqual(m.snippet.gl, "DZ")

    def testI18nRegionResponse(self) -> None:
        m = models.I18nRegionListResponse.from_dict(self.REGION_RES)

        self.assertEqual(m.kind, "youtube#i18nRegionListResponse")
        self.assertEqual(len(m.items), 2)

    def testI18nLanguage(self) -> None:
        m = models.I18nLanguage.from_dict(self.LANGUAGE_INFO)

        self.assertEqual(m.id, "af")
        self.assertEqual(m.snippet.hl, "af")

    def testI18nLanguageResponse(self) -> None:
        m = models.I18nRegionListResponse.from_dict(self.LANGUAGE_RES)

        self.assertEqual(m.kind, "youtube#i18nLanguageListResponse")
        self.assertEqual(len(m.items), 2)



import json
import unittest

import responses

import pyyoutube


class ApiPlaylistTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/playlists/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/playlists"

    with open(BASE_PATH + "playlists_single.json", "rb") as f:
        PLAYLISTS_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlists_multi.json", "rb") as f:
        PLAYLISTS_INFO_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlists_paged_1.json", "rb") as f:
        PLAYLISTS_PAGED_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlists_paged_2.json", "rb") as f:
        PLAYLISTS_PAGED_2 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "playlists_mine.json", "rb") as f:
        PLAYLISTS_MINE = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")
        self.api_with_access_token = pyyoutube.Api(access_token="access token")

    def testGetPlaylistById(self) -> None:
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_INFO_SINGLE)
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_INFO_MULTI)

            res_by_playlist_id = self.api.get_playlist_by_id(
                playlist_id="PLOU2XLYxmsIJXsH2htG1g0NUjHGq62Q7i",
                parts="id,snippet",
                return_json=True,
            )
            self.assertEqual(res_by_playlist_id["kind"], "youtube#playlistListResponse")
            self.assertEqual(
                res_by_playlist_id["items"][0]["id"],
                "PLOU2XLYxmsIJXsH2htG1g0NUjHGq62Q7i",
            )

            res_by_playlist_multi_id = self.api.get_playlist_by_id(
                playlist_id=[
                    "PLOU2XLYxmsIJXsH2htG1g0NUjHGq62Q7i",
                    "PLOU2XLYxmsIJJVnHWmd1qfr0Caq4VZCu4",
                ],
                parts=["id", "snippet"],
            )
            self.assertEqual(len(res_by_playlist_multi_id.items), 2)
            self.assertEqual(
                res_by_playlist_multi_id.items[1].id,
                "PLOU2XLYxmsIJJVnHWmd1qfr0Caq4VZCu4",
            )

    def testGetPlaylists(self) -> None:
        # test params checker
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_playlists(parts="id,not_part")
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_playlists()

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_PAGED_2)
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_MINE)

            res_by_channel_id = self.api.get_playlists(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                limit=10,
                count=13,
            )
            self.assertEqual(res_by_channel_id.pageInfo.totalResults, 422)
            self.assertEqual(len(res_by_channel_id.items), 13)
            self.assertEqual(
                res_by_channel_id.items[0].snippet.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw"
            )

            res_by_mine = self.api_with_access_token.get_playlists(
                mine=True, limit=10, count=10, return_json=True
            )
            self.assertEqual(len(res_by_mine["items"]), 2)
            self.assertEqual(
                res_by_mine["items"][0]["id"], "PLOU2XLYxmsIIOSO0eWuj-6yQmdakarUzN"
            )

        # test for all items
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_PAGED_1)
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_PAGED_2)

            res_by_channel_id = self.api.get_playlists(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                count=None,
            )
            self.assertEqual(len(res_by_channel_id.items), 20)

        # test for page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.PLAYLISTS_PAGED_2)

            res_by_channel_id = self.api.get_playlists(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw", count=None, page_token="CAoQAA"
            )
            self.assertEqual(len(res_by_channel_id.items), 10)



import json
import unittest

import responses
import pyyoutube


class ApiVideoAbuseReason(unittest.TestCase):
    BASE_PATH = "testdata/apidata/abuse_reasons/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/videoAbuseReportReasons"

    with open(BASE_PATH + "abuse_reason.json", "rb") as f:
        ABUSE_REASON_RES = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api_with_token = pyyoutube.Api(access_token="access token")

    def testGetVideoAbuseReportReason(self) -> None:
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.ABUSE_REASON_RES)

            abuse_res = self.api_with_token.get_video_abuse_report_reason(
                parts=["id", "snippet"],
            )

            self.assertEqual(
                abuse_res.kind, "youtube#videoAbuseReportReasonListResponse"
            )
            self.assertEqual(len(abuse_res.items), 3)

            abuse_res_json = self.api_with_token.get_video_abuse_report_reason(
                return_json=True
            )

            self.assertEqual(len(abuse_res_json["items"]), 3)



import json
import unittest
import pyyoutube
import pyyoutube.models as models


class VideoModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/videos/"

    with open(BASE_PATH + "video_content_details.json", "rb") as f:
        CONTENT_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_topic_details.json", "rb") as f:
        TOPIC_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_snippet.json", "rb") as f:
        SNIPPET_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_statistics.json", "rb") as f:
        STATISTICS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_status.json", "rb") as f:
        STATUS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_info.json", "rb") as f:
        VIDEO_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_api_response.json", "rb") as f:
        VIDEO_API_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testVideoContentDetails(self) -> None:
        m = models.VideoContentDetails.from_dict(self.CONTENT_DETAILS_INFO)

        self.assertEqual(m.duration, "PT21M7S")

        seconds = m.get_video_seconds_duration()
        self.assertEqual(seconds, 1267)

        m.duration = None
        self.assertEqual(m.get_video_seconds_duration(), None)

        with self.assertRaises(pyyoutube.PyYouTubeException):
            m.duration = "error datetime"
            m.get_video_seconds_duration()

    def testVideoTopicDetails(self) -> None:
        m = models.VideoTopicDetails.from_dict(self.TOPIC_DETAILS_INFO)

        self.assertEqual(m.topicIds[0], "/m/02jjt")
        self.assertEqual(len(m.topicCategories), 1)

        full_topics = m.get_full_topics()

        self.assertEqual(full_topics[0].id, "/m/02jjt")
        self.assertEqual(full_topics[0].description, "Entertainment (parent topic)")

    def testVideoSnippet(self) -> None:
        m = models.VideoSnippet.from_dict(self.SNIPPET_INFO)

        self.assertEqual(
            m.string_to_datetime(m.publishedAt).isoformat(), "2019-03-21T20:37:49+00:00"
        )

        m.publishedAt = None
        self.assertEqual(m.string_to_datetime(m.publishedAt), None)

        with self.assertRaises(pyyoutube.PyYouTubeException):
            m.string_to_datetime("error datetime string")

        self.assertEqual(m.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw")
        self.assertEqual(
            m.thumbnails.default.url, "https://i.ytimg.com/vi/D-lhorsDlUQ/default.jpg"
        )
        self.assertEqual(m.tags[0], "Google")
        self.assertEqual(
            m.localized.title, "What are Actions on Google (Assistant on Air)"
        )

    def testVideoStatistics(self) -> None:
        m = models.VideoStatistics.from_dict(self.STATISTICS_INFO)

        self.assertEqual(m.viewCount, "8087")

    def testVideoStatus(self) -> None:
        m = models.VideoStatus.from_dict(self.STATUS_INFO)

        self.assertEqual(m.uploadStatus, "processed")

        self.assertEqual(
            m.string_to_datetime(m.publishAt).isoformat(), "2019-03-21T20:37:49+00:00"
        )

    def testVideo(self) -> None:
        m = models.Video.from_dict(self.VIDEO_INFO)

        self.assertEqual(m.id, "D-lhorsDlUQ")
        self.assertEqual(
            m.snippet.title, "What are Actions on Google (Assistant on Air)"
        )

    def testVideoListResponse(self) -> None:
        m = models.VideoListResponse.from_dict(self.VIDEO_API_RESPONSE)
        self.assertEqual(m.kind, "youtube#videoListResponse")
        self.assertEqual(m.pageInfo.totalResults, 1)
        self.assertEqual(m.items[0].id, "D-lhorsDlUQ")



import json
import unittest

import responses
import pyyoutube


class ApiCommentThreadTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/comment_threads/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/commentThreads"

    with open(BASE_PATH + "comment_thread_single.json", "rb") as f:
        COMMENT_THREAD_INFO_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_multi.json", "rb") as f:
        COMMENT_THREAD_INFO_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_all_to_me.json", "rb") as f:
        COMMENT_THREAD_ALL_TO_ME = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_by_channel.json", "rb") as f:
        COMMENT_THREAD_BY_CHANNEL = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_with_search.json", "rb") as f:
        COMMENT_THREAD_BY_SEARCH = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_by_video_paged_1.json", "rb") as f:
        COMMENT_THREAD_BY_VIDEO_P_1 = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "comment_threads_by_video_paged_2.json", "rb") as f:
        COMMENT_THREAD_BY_VIDEO_P_2 = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")
        self.api_with_token = pyyoutube.Api(access_token="access token")

    def testGetCommentThreadById(self) -> None:
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_comment_thread_by_id(
                comment_thread_id="id", parts="id,not_part"
            )

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_INFO_SINGLE)
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_INFO_MULTI)

            res_by_single_id = self.api.get_comment_thread_by_id(
                comment_thread_id="UgxKREWxIgDrw8w2e_Z4AaABAg",
                parts="id,snippet",
                text_format="plain_text",
                return_json=True,
            )
            self.assertEqual(
                res_by_single_id["kind"], "youtube#commentThreadListResponse"
            )
            self.assertEqual(len(res_by_single_id["items"]), 1)
            self.assertEqual(
                res_by_single_id["items"][0]["id"], "UgxKREWxIgDrw8w2e_Z4AaABAg"
            )

            res_by_multi_id = self.api.get_comment_thread_by_id(
                comment_thread_id=[
                    "UgxKREWxIgDrw8w2e_Z4AaABAg",
                    "UgyrVQaFfEdvaSzstj14AaABAg",
                ],
                parts=["id", "snippet"],
            )
            self.assertEqual(res_by_multi_id.pageInfo.totalResults, 2)
            self.assertEqual(res_by_multi_id.items[1].id, "UgyrVQaFfEdvaSzstj14AaABAg")

    def testGetCommentThreads(self) -> None:
        # test no params
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_comment_threads()
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_comment_threads(all_to_channel_id="id", parts="id,not_part")

        # test with all to channel.
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_ALL_TO_ME)

            res_by_all = self.api_with_token.get_comment_threads(
                all_to_channel_id="UCa-vrCLQHviTOVnEKDOdetQ",
                parts="id,snippet",
                moderation_status="published",
                order="time",
                return_json=True,
            )
            self.assertEqual(res_by_all["kind"], "youtube#commentThreadListResponse")
            self.assertEqual(res_by_all["pageInfo"]["totalResults"], 4)
            self.assertEqual(len(res_by_all["items"]), 4)
            self.assertEqual(res_by_all["items"][0]["id"], "UgyWeTdgc4sc1xgmbld4AaABAg")

        # test with channel
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_CHANNEL)

            res_by_channel = self.api.get_comment_threads(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
            )
            self.assertEqual(res_by_channel.pageInfo.totalResults, 2)
            self.assertEqual(
                res_by_channel.items[0].snippet.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw"
            )

        # test with search
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_SEARCH)

            res_by_search = self.api.get_comment_threads(
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                search_terms="Hello",
            )
            self.assertEqual(res_by_search.pageInfo.totalResults, 1)
            self.assertEqual(
                res_by_channel.items[0].snippet.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw"
            )
            self.assertIn(
                "Hello",
                res_by_channel.items[0].snippet.topLevelComment.snippet.textDisplay,
            )

        # test with video
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_VIDEO_P_1)
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_VIDEO_P_2)

            res_by_video = self.api.get_comment_threads(
                video_id="F1UP7wRCPH8",
                count=8,
                limit=5,
            )
            self.assertEqual(len(res_by_video.items), 8)
            self.assertEqual(res_by_video.items[0].snippet.videoId, "F1UP7wRCPH8")

        # test get all items
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_VIDEO_P_1)
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_VIDEO_P_2)

            res_by_video = self.api.get_comment_threads(
                video_id="F1UP7wRCPH8",
                count=None,
            )
            self.assertEqual(len(res_by_video.items), 10)

        # test use page token
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.COMMENT_THREAD_BY_VIDEO_P_2)

            res_by_video = self.api.get_comment_threads(
                video_id="F1UP7wRCPH8",
                count=None,
                page_token="QURTSl9pMzdZOUVzMkI0czlmRmNjSVBPcTBTdzVzajUydDVnbE5SNElWS0l5WU12amYweVotdzF5c1hTNmxzUmVIcEZXbmVEVFMzNVJmWk82TVVwUlB2LWh5aUpOQlA5TGQzTWZEcHlTeTd2dlNGRUFZaVF0cmtJd01BTHlnOG0=",
            )
            self.assertEqual(len(res_by_video.items), 5)



import json
import unittest

import responses

import pyyoutube


class ApiSearchTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/search/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/search"

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testSearch(self) -> None:
        with open(self.BASE_PATH + "search_videos_by_channel.json", "rb") as f:
            search_videos_by_channel = json.loads(f.read().decode("utf-8"))
        with open(self.BASE_PATH + "search_by_location.json", "rb") as f:
            search_by_location = json.loads(f.read().decode("utf-8"))
        with open(self.BASE_PATH + "search_by_event.json", "rb") as f:
            search_by_event = json.loads(f.read().decode("utf-8"))
        with open(self.BASE_PATH + "search_channels.json", "rb") as f:
            search_channels = json.loads(f.read().decode("utf-8"))

        # test search videos with channel
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_videos_by_channel)

            res = self.api.search(
                parts="snippet",
                channel_id="UC_x5XG1OV2P6uZZ5FSM9Ttw",
                q="news",
                count=5,
            )
            self.assertEqual(res.items[0].id.videoId, "LrQWzOkC0XQ")
            self.assertEqual(res.items[0].snippet.channelId, "UC_x5XG1OV2P6uZZ5FSM9Ttw")

        # test search locations
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_by_location)
            res = self.api.search(
                location="21.5922529, -158.1147114",
                location_radius="10mi",
                q="surfing",
                parts=["snippet"],
                count=5,
                published_after="2020-02-01T00:00:00Z",
                published_before="2020-03-01T00:00:00Z",
                safe_search="moderate",
                search_type="video",
            )
            self.assertEqual(res.pageInfo.resultsPerPage, 5)
            self.assertEqual(len(res.items), 5)
            self.assertEqual(res.items[0].snippet.channelId, "UCo_q6aOlvPH7M-j_XGWVgXg")

        # test search event
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_by_event)
            res = self.api.search(
                event_type="live",
                q="news",
                count=25,
                limit=25,
                parts=["snippet"],
                search_type="video",
                topic_id="/m/09s1f",
                order="viewCount",
            )

            self.assertEqual(res.pageInfo.resultsPerPage, 25)
            self.assertEqual(len(res.items), 25)
            self.assertEqual(res.items[0].snippet.channelId, "UCDGiCfCZIV5phsoGiPwIcyQ")

        # test search channel
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_channels)

            res_channels = self.api.search(
                parts=["snippet"],
                channel_type="any",
                count=5,
                search_type="channel",
            )
            self.assertEqual(res_channels.pageInfo.resultsPerPage, 5)
            self.assertEqual(
                res_channels.items[0].snippet.channelId, "UCxRULEz6kS0PMxCzOY25GhQ"
            )

    def testSearchByKeywords(self) -> None:
        with open(self.BASE_PATH + "search_by_keywords_p1.json", "rb") as f:
            res_p1 = json.loads(f.read().decode("utf-8"))
        with open(self.BASE_PATH + "search_by_keywords_p2.json", "rb") as f:
            res_p2 = json.loads(f.read().decode("utf-8"))

        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.search_by_keywords(q="x", parts="id,not_part")

        # test response
        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=res_p1)
            m.add("GET", self.BASE_URL, json=res_p2)

            res_json = self.api.search_by_keywords(
                q="surfing", count=30, limit=25, return_json=True
            )
            self.assertEqual(res_json["kind"], "youtube#searchListResponse")
            self.assertEqual(res_json["regionCode"], "JP")
            self.assertEqual(res_json["pageInfo"]["totalResults"], 1000000)
            self.assertEqual(len(res_json["items"]), 30)

            res = self.api.search_by_keywords(
                q="surfing",
                parts=["id", "snippet"],
                count=25,
            )
            self.assertEqual(res.pageInfo.resultsPerPage, 25)
            self.assertEqual(res.items[0].id.videoId, "-2IlD-x8wvY")
            self.assertEqual(res.items[0].snippet.channelId, "UCeYue9Nbodzg3T1Nt88E3fg")

    def testSearchByRelatedToVideoId(self) -> None:
        with open(self.BASE_PATH + "search_by_related_video.json", "rb") as f:
            search_by_related_video = json.loads(f.read().decode("utf-8"))

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_by_related_video)

            res = self.api.search_by_related_video(
                related_to_video_id="Ks-_Mh1QhMc",
                region_code="US",
                relevance_language="en",
                safe_search="moderate",
                count=5,
            )

            self.assertEqual(res.pageInfo.resultsPerPage, 5)
            self.assertEqual(len(res.items), 5)
            self.assertEqual(res.regionCode, "US")
            self.assertEqual(res.items[0].snippet.channelId, "UCAuUUnT6oDeKwE6v1NGQxug")

    def testSearchByDeveloper(self) -> None:
        with open(self.BASE_PATH + "search_by_developer.json", "rb") as f:
            search_by_developer = json.loads(f.read().decode("utf-8"))

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_by_developer)

            res_dev = self.api.search_by_developer(
                parts=["snippet"],
                q="news",
                count=5,
                page_token="CAUQAA",
                video_category_id="17",
                video_caption="any",
                video_definition="any",
                video_dimension="any",
                video_duration="any",
                video_embeddable="any",
                video_license="any",
                video_syndicated="any",
                video_type="any",
            )
            self.assertEqual(res_dev.pageInfo.resultsPerPage, 5)
            self.assertEqual(
                res_dev.items[0].snippet.channelId, "UCeY0bbntWzzVIaj2z3QigXg"
            )

    def testSearchByMine(self) -> None:
        with open(self.BASE_PATH + "search_by_mine.json", "rb") as f:
            search_by_mine = json.loads(f.read().decode("utf-8"))

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=search_by_mine)

            res_mine = self.api.search_by_mine(
                parts=["snippet"],
            )

            self.assertEqual(res_mine.pageInfo.totalResults, 2)
            self.assertEqual(
                res_mine.items[0].snippet.channelId, "UCa-vrCLQHviTOVnEKDOdetQ"
            )



import json
import unittest
import pyyoutube.models as models


class ChannelModelTest(unittest.TestCase):
    BASE_PATH = "testdata/modeldata/channels/"

    with open(BASE_PATH + "channel_branding_settings.json", "rb") as f:
        BRANDING_SETTINGS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_content_details.json", "rb") as f:
        CONTENT_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_topic_details.json", "rb") as f:
        TOPIC_DETAILS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_snippet.json", "rb") as f:
        SNIPPET_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_statistics.json", "rb") as f:
        STATISTICS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_status.json", "rb") as f:
        STATUS_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_info.json", "rb") as f:
        CHANNEL_INFO = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "channel_api_response.json", "rb") as f:
        CHANNEL_API_RESPONSE = json.loads(f.read().decode("utf-8"))

    def testChannelBrandingSettings(self) -> None:
        m = models.ChannelBrandingSetting.from_dict(self.BRANDING_SETTINGS_INFO)

        self.assertEqual(m.channel.title, "Google Developers")

    def testChannelContentDetails(self) -> None:
        m = models.ChannelContentDetails.from_dict(self.CONTENT_DETAILS_INFO)

        self.assertEqual(m.relatedPlaylists.uploads, "UU_x5XG1OV2P6uZZ5FSM9Ttw")

    def testChannelTopicDetails(self) -> None:
        m = models.ChannelTopicDetails.from_dict(self.TOPIC_DETAILS_INFO)

        self.assertEqual(m.topicIds[0], "/m/019_rr")
        self.assertEqual(len(m.topicCategories), 3)

        full_topics = m.get_full_topics()
        self.assertEqual(full_topics[0].id, "/m/019_rr")
        self.assertEqual(full_topics[0].description, "Lifestyle (parent topic)")

    def testChannelSnippet(self) -> None:
        m = models.ChannelSnippet.from_dict(self.SNIPPET_INFO)

        self.assertEqual(m.title, "Google Developers")
        self.assertEqual(m.localized.title, "Google Developers")
        self.assertEqual(
            m.thumbnails.default.url,
            "https://yt3.ggpht.com/a/AGF-l78iFtAxyRZcUBzG91kbKMES19z-zGW5KT20_g=s88-c-k-c0xffffffff-no-rj-mo",
        )

        published_at = m.string_to_datetime(m.publishedAt)
        self.assertEqual(published_at.isoformat(), "2007-08-23T00:34:43+00:00")

    def testChannelStatistics(self) -> None:
        m = models.ChannelStatistics.from_dict(self.STATISTICS_INFO)

        self.assertEqual(m.viewCount, "160361638")

    def testChannelStatus(self) -> None:
        m = models.ChannelStatus.from_dict(self.STATUS_INFO)

        self.assertEqual(m.privacyStatus, "public")

    def testChannel(self) -> None:
        m = models.Channel.from_dict(self.CHANNEL_INFO)

        self.assertEqual(m.id, "UC_x5XG1OV2P6uZZ5FSM9Ttw")

    def testChannelListResponse(self) -> None:
        m = models.ChannelListResponse.from_dict(self.CHANNEL_API_RESPONSE)

        self.assertEqual(m.kind, "youtube#channelListResponse")
        self.assertEqual(m.pageInfo.totalResults, 1)
        self.assertEqual(m.items[0].id, "UC_x5XG1OV2P6uZZ5FSM9Ttw")



"""
    This provide some common utils methods for YouTube resource.
"""

import isodate
from isodate.isoerror import ISO8601Error

from pyyoutube.error import ErrorMessage, PyYouTubeException


def get_video_duration(duration: str) -> int:
    """
    Parse video ISO 8601 duration to seconds.
    Refer: https://developers.google.com/youtube/v3/docs/videos#contentDetails.duration

    Args:
        duration(str)
            Videos ISO 8601 duration. Like: PT14H23M42S
    Returns:
        integer for seconds.
    """
    try:
        seconds = isodate.parse_duration(duration).total_seconds()
        return int(seconds)
    except ISO8601Error as e:
        raise PyYouTubeException(
            ErrorMessage(
                status_code=10001,
                message=f"Exception in convert video duration: {duration}. errors: {e}",
            )
        )



import json
import unittest

import responses

import pyyoutube


class ApiVideoCategoryTest(unittest.TestCase):
    BASE_PATH = "testdata/apidata/categories/"
    BASE_URL = "https://www.googleapis.com/youtube/v3/videoCategories"

    with open(BASE_PATH + "video_category_single.json", "rb") as f:
        VIDEO_CATEGORY_SINGLE = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_category_multi.json", "rb") as f:
        VIDEO_CATEGORY_MULTI = json.loads(f.read().decode("utf-8"))
    with open(BASE_PATH + "video_category_by_region.json", "rb") as f:
        VIDEO_CATEGORY_BY_REGION = json.loads(f.read().decode("utf-8"))

    def setUp(self) -> None:
        self.api = pyyoutube.Api(api_key="api key")

    def testGetVideoCategories(self) -> None:
        # test params
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_video_categories()
        # test parts
        with self.assertRaises(pyyoutube.PyYouTubeException):
            self.api.get_video_categories(category_id="id", parts="id,not_part")

        with responses.RequestsMock() as m:
            m.add("GET", self.BASE_URL, json=self.VIDEO_CATEGORY_SINGLE)
            m.add("GET", self.BASE_URL, json=self.VIDEO_CATEGORY_MULTI)
            m.add("GET", self.BASE_URL, json=self.VIDEO_CATEGORY_BY_REGION)

            res_by_single = self.api.get_video_categories(
                category_id="17",
                parts=["id", "snippet"],
                return_json=True,
            )
            self.assertEqual(res_by_single["kind"], "youtube#videoCategoryListResponse")
            self.assertEqual(len(res_by_single["items"]), 1)
            self.assertEqual(res_by_single["items"][0]["id"], "17")

            res_by_multi = self.api.get_video_categories(
                category_id=["17", "18"],
                parts="id,snippet",
            )
            self.assertEqual(len(res_by_multi.items), 2)
            self.assertEqual(res_by_multi.items[1].id, "18")

            res_by_region = self.api.get_video_categories(
                region_code="US",
                parts="id,snippet",
            )
            self.assertEqual(len(res_by_region.items), 32)
            self.assertEqual(res_by_region.items[0].id, "1")
